@@ -114,14 +114,15 @@ def bsearch_py(times, L, m, T, slice_=None,
             if not above_min_rate_:
                 break
         i_stop = i + m - 2  # index of last ph in burst
-
-        bursts.append((i_start, i_stop, times[i_start], times[i_stop]))
+        if i_stop - i_start + 1 >= L:
+            bursts.append((i_start, i_stop, times[i_start], times[i_stop]))
 
     if above_min_rate_:
         # Correct burst-stop off by 1 when last burst does not finish
-        i_start, i_stop, start, stop = bursts.pop()
-        bursts.append((i_start, i_stop + 1,
-                       times[i_start], times[i_stop + 1]))
+        i_stop += 1
+        if i_stop - i_start + 1 >= L:
+            bursts.pop()
+            bursts.append((i_start, i_stop, times[i_start], times[i_stop]))
 
     bursts = np.array(bursts, dtype='int64')
     if bursts.size > 0:
@@ -267,7 +268,7 @@ class Bursts(object):
     stop) such as the ones returned by burst search functions (e.g.
     :func:`bsearch_py`).
 
-    `Bursts` object are iterable, yielding one burst a time (:class:`Burst`
+    `Bursts` objects are iterable, yielding one burst a time (:class:`Burst`
     objects). `Bursts` can be compared for equality (with `==`) and copied
     (:meth:`Bursts.copy`).
 
