@@ -15,17 +15,32 @@ from builtins import range, zip
 from collections import namedtuple
 import pytest
 import numpy as np
-import matplotlib
-matplotlib.use('Agg')
+
+try:
+    import matplotlib
+except ImportError:
+    has_matplotlib = False  # OK to run tests without matplotlib
+else:
+    has_matplotlib = True
+    matplotlib.use('Agg')  # but if matplotlib is installed, use Agg
+
+try:
+    import numba
+except ImportError:
+    has_numba = False
+else:
+    has_numba = True
+
 
 import fretbursts.background as bg
 import fretbursts.burstlib as bl
 import fretbursts.burstlib_ext as bext
-import fretbursts.burst_plot as bplt
 from fretbursts import loader
 from fretbursts import select_bursts
 from fretbursts.ph_sel import Ph_sel
 from fretbursts.phtools import phrates
+if has_matplotlib:
+    import fretbursts.burst_plot as bplt
 
 
 # data subdir in the notebook folder
@@ -97,7 +112,6 @@ def list_array_allclose(list1, list2):
 # Test functions
 #
 
-
 def test_bg_compatlayer_for_obsolete_attrs():
     d = load_dataset_1ch(process=False)
     attrs = ('bg_dd', 'bg_ad', 'bg_da', 'bg_aa',
@@ -179,7 +193,7 @@ def test_bg_calc(data):
 
 def test_ph_streams(data):
     sel = [Ph_sel('all'), Ph_sel(Dex='Dem'), Ph_sel(Dex='Aem')]
-    if data.ALEX:
+    if data.alternated:
         sel.extend([Ph_sel(Aex='Aem'), Ph_sel(Aex='Dem')])
     for s in sel:
         assert s in data.ph_streams
@@ -193,7 +207,7 @@ def test_bg_from(data):
         bg = d.bg_from(ph_sel=sel)
         assert list_array_equal(bg, d.bg[sel])
 
-    if not data.ALEX:
+    if not (data.alternated):
         assert list_array_equal(d.bg_from(Ph_sel('all')),
                                 d.bg_from(Ph_sel(Dex='DAem')))
         return
@@ -233,18 +247,18 @@ def test_iter_ph_times(data):
     assert list_array_equal(d.ph_times_m, d.iter_ph_times())
 
     for ich, ph in enumerate(d.iter_ph_times(Ph_sel(Dex='Dem'))):
-        if d.ALEX:
+        if d.alternated:
             assert (ph == d.ph_times_m[ich][d.D_em[ich] * d.D_ex[ich]]).all()
         else:
-            assert (ph == d.ph_times_m[ich][-d.A_em[ich]]).all()
+            assert (ph == d.ph_times_m[ich][~d.A_em[ich]]).all()
 
     for ich, ph in enumerate(d.iter_ph_times(Ph_sel(Dex='Aem'))):
-        if d.ALEX:
+        if d.alternated:
             assert (ph == d.ph_times_m[ich][d.A_em[ich] * d.D_ex[ich]]).all()
         else:
             assert (ph == d.ph_times_m[ich][d.A_em[ich]]).all()
 
-    if d.ALEX:
+    if d.alternated:
         for ich, ph in enumerate(d.iter_ph_times(Ph_sel(Aex='Dem'))):
             assert (ph == d.ph_times_m[ich][d.D_em[ich] * d.A_ex[ich]]).all()
         for ich, ph in enumerate(d.iter_ph_times(Ph_sel(Aex='Aem'))):
@@ -332,45 +346,47 @@ def test_burst_search_with_no_bursts(data):
     # F=600 results in periods with no bursts for the us-ALEX measurement
     # and in no bursts at all for the multi-spot measurements
     data.burst_search(m=10, F=600)
+    data.fuse_bursts(ms=1)
 
-def test_stale_fitter_after_burst_search(data):
-    """Test that E/S_fitter attributes are deleted on burst search."""
-    data.burst_search(L=10, m=10, F=7, ph_sel=Ph_sel(Dex='Dem'))
-    bplt.dplot(data, bplt.hist_fret)  # create E_fitter attribute
-    if data.ALEX:
-        bplt.dplot(data, bplt.hist_S)  # create S_fitter attribute
+if has_matplotlib:
+    def test_stale_fitter_after_burst_search(data):
+        """Test that E/S_fitter attributes are deleted on burst search."""
+        data.burst_search(L=10, m=10, F=7, ph_sel=Ph_sel(Dex='Dem'))
+        bplt.dplot(data, bplt.hist_fret)  # create E_fitter attribute
+        if data.alternated:
+            bplt.dplot(data, bplt.hist_S)  # create S_fitter attribute
 
-    data.burst_search(L=10, m=10, F=7, ph_sel=Ph_sel(Dex='Aem'))
-    assert not hasattr(data, 'E_fitter')
-    if data.ALEX:
-        assert not hasattr(data, 'S_fitter')
+        data.burst_search(L=10, m=10, F=7, ph_sel=Ph_sel(Dex='Aem'))
+        assert not hasattr(data, 'E_fitter')
+        if data.alternated:
+            assert not hasattr(data, 'S_fitter')
 
-    bplt.dplot(data, bplt.hist_fret)  # create E_fitter attribute
-    if data.ALEX:
-        bplt.dplot(data, bplt.hist_S)  # create S_fitter attribute
+        bplt.dplot(data, bplt.hist_fret)  # create E_fitter attribute
+        if data.alternated:
+            bplt.dplot(data, bplt.hist_S)  # create S_fitter attribute
 
-    data.calc_fret()
-    assert not hasattr(data, 'E_fitter')
-    if data.ALEX:
-        assert not hasattr(data, 'S_fitter')
+        data.calc_fret()
+        assert not hasattr(data, 'E_fitter')
+        if data.alternated:
+            assert not hasattr(data, 'S_fitter')
 
 def test_burst_search(data):
     """Smoke test and bg_bs check."""
     streams = [Ph_sel(Dex='Dem'), Ph_sel(Dex='Aem')]
-    if data.ALEX:
+    if data.alternated:
         streams.extend([Ph_sel(Dex='Aem', Aex='Aem'), Ph_sel(Dex='DAem')])
     for sel in streams:
         data.burst_search(L=10, m=10, F=7, ph_sel=sel)
         assert list_equal(data.bg_bs, data.bg_from(sel))
 
-    if data.ALEX:
+    if data.alternated:
         data.burst_search(m=10, F=7, ph_sel=Ph_sel(Dex='DAem'), compact=True)
     data.burst_search(L=10, m=10, F=7)
 
 def test_burst_search_and_gate(data_1ch):
     """Test consistency of burst search and gate."""
     d = data_1ch
-    assert d.ALEX
+    assert d.alternated
     d_dex = d.copy()
     d_dex.burst_search(ph_sel=Ph_sel(Dex='DAem'))
     d_aex = d.copy()
@@ -383,6 +399,7 @@ def test_burst_search_and_gate(data_1ch):
         ph_b_mask_and = bl.ph_in_bursts_mask(ph.size, bursts_and)
         assert (ph_b_mask_and == ph_b_mask_dex * ph_b_mask_aex).all()
 
+
 def test_mch_count_ph_num_py_c(data):
     na_py = bl.bslib.mch_count_ph_in_bursts_py(data.mburst, data.A_em)
     na_c = bl.bslib.mch_count_ph_in_bursts_c(data.mburst, data.A_em)
@@ -399,10 +416,10 @@ def test_burst_sizes(data):
     bs2 = data.burst_sizes_ich(gamma=0.5, donor_ref=False)
     assert np.allclose(bs1, bs2 / 0.5)
     # Test add_naa
-    if data.ALEX:
+    if data.alternated:
         bs_no_naa = data.burst_sizes_ich(add_naa=False)
         bs_naa = data.burst_sizes_ich(add_naa=True)
-        assert np.allclose(bs_no_naa + data.naa_, bs_naa)
+        assert np.allclose(bs_no_naa + data.naa[0], bs_naa)
 
         # Test beta and donor_ref arguments with gamma=1
         naa1 = data.get_naa_corrected(beta=0.8, donor_ref=True)
@@ -598,44 +615,45 @@ def test_phrates_mtuple(data):
         assert (rates == (m - 1 - phrates.default_c) / delays).all()
         assert (phrates.mtuple_rates_t(phc, m) == t_rates).all()
 
-def test_phrates_kde(data):
-    d = data
-    tau = 5000  # 5000 * 12.5ns = 6.25 us
-    for ph in d.iter_ph_times():
-        # Test consistency of kde_laplace_nph and (kde_laplace, kde_rect)
-        rates = phrates.kde_laplace(ph, tau)
-        nrect = phrates.kde_rect(ph, tau*10)
-        ratesl, nph = phrates.nb.kde_laplace_nph(ph, tau)
-        assert (rates == ratesl).all()
-        assert (nph == nrect).all()
+if has_numba:
+    def test_phrates_kde(data):
+        d = data
+        tau = 5000  # 5000 * 12.5ns = 6.25 us
+        for ph in d.iter_ph_times():
+            # Test consistency of kde_laplace_nph and (kde_laplace, kde_rect)
+            rates = phrates.kde_laplace(ph, tau)
+            nrect = phrates.kde_rect(ph, tau*10)
+            ratesl, nph = phrates.nb.kde_laplace_nph(ph, tau)
+            assert (rates == ratesl).all()
+            assert (nph == nrect).all()
 
-        # Test consistency of kde_laplace and _kde_laplace_self_numba
-        ratesl2, nph2 = phrates.nb.kde_laplace_self_numba(ph, tau)
-        assert (nph2 == nrect).all()
-        assert (ratesl2 == rates).all()
+            # Test consistency of kde_laplace and _kde_laplace_self_numba
+            ratesl2, nph2 = phrates.nb.kde_laplace_self_numba(ph, tau)
+            assert (nph2 == nrect).all()
+            assert (ratesl2 == rates).all()
 
-        # Smoke test laplace, gaussian, rect with time_axis
-        ratesl = phrates.kde_laplace(ph, tau, time_axis=ph+1)
-        assert ((ratesl >= 0) * (ratesl < 5e6)).all()
-        ratesg = phrates.kde_gaussian(ph, tau, time_axis=ph+1)
-        assert ((ratesg >= 0) * (ratesg < 5e6)).all()
-        ratesr = phrates.kde_rect(ph, tau, time_axis=ph+1)
-        assert ((ratesr >= 0) * (ratesr < 5e6)).all()
+            # Smoke test laplace, gaussian, rect with time_axis
+            ratesl = phrates.kde_laplace(ph, tau, time_axis=ph+1)
+            assert ((ratesl >= 0) * (ratesl < 5e6)).all()
+            ratesg = phrates.kde_gaussian(ph, tau, time_axis=ph+1)
+            assert ((ratesg >= 0) * (ratesg < 5e6)).all()
+            ratesr = phrates.kde_rect(ph, tau, time_axis=ph+1)
+            assert ((ratesr >= 0) * (ratesr < 5e6)).all()
 
-def test_phrates_kde_cy(data):
-    d = data
-    tau = 5000  # 5000 * 12.5ns = 6.25 us
-    for ph in d.iter_ph_times():
-        # Test consistency of kde_laplace_nph and (kde_laplace, kde_rect)
-        ratesg = phrates.nb.kde_gaussian_numba(ph, tau)
-        ratesl = phrates.nb.kde_laplace_numba(ph, tau)
-        ratesr = phrates.nb.kde_rect_numba(ph, tau)
-        ratesgc = phrates.cy.kde_gaussian_cy(ph, tau)
-        rateslc = phrates.cy.kde_laplace_cy(ph, tau)
-        ratesrc = phrates.cy.kde_rect_cy(ph, tau)
-        assert (ratesg == ratesgc).all()
-        assert (ratesl == rateslc).all()
-        assert (ratesr == ratesrc).all()
+    def test_phrates_kde_cy(data):
+        d = data
+        tau = 5000  # 5000 * 12.5ns = 6.25 us
+        for ph in d.iter_ph_times():
+            # Test consistency of kde_laplace_nph and (kde_laplace, kde_rect)
+            ratesg = phrates.nb.kde_gaussian_numba(ph, tau)
+            ratesl = phrates.nb.kde_laplace_numba(ph, tau)
+            ratesr = phrates.nb.kde_rect_numba(ph, tau)
+            ratesgc = phrates.cy.kde_gaussian_cy(ph, tau)
+            rateslc = phrates.cy.kde_laplace_cy(ph, tau)
+            ratesrc = phrates.cy.kde_rect_cy(ph, tau)
+            assert (ratesg == ratesgc).all()
+            assert (ratesl == rateslc).all()
+            assert (ratesr == ratesrc).all()
 
 def test_burst_ph_data_functions(data):
     """Tests the functions that iterate or operate on per-burst "ph-data".
@@ -698,11 +716,18 @@ def test_burst_fuse_0ms(data):
     as the number of ph in bursts (via burst selection).
     """
     d = data
-
+    if d.nch == 8:
+        d.burst_search(L=10, m=10, F=7, computefret=False)
+        d.mburst[1] = bl.bslib.Bursts.empty()  # Make one channel with no bursts
+        d._calc_burst_period()
+        d.calc_fret(count_ph=True)
     df = d.fuse_bursts(ms=0)
     for ich, bursts in enumerate(df.mburst):
         mask = bl.ph_in_bursts_mask(df.ph_data_sizes[ich], bursts)
         assert mask.sum() == bursts.counts.sum()
+    df.calc_fret(count_ph=True)
+    assert len(df.mburst) == len(d.mburst)
+    assert len(df.mburst) == d.nch
 
 def test_burst_fuse_separation(data):
     """Test that after fusing bursts the minimum separation is equal
@@ -712,8 +737,9 @@ def test_burst_fuse_separation(data):
     fuse_ms = 2
     df = d.fuse_bursts(ms=fuse_ms)
     for bursts in df.mburst:
-        separation = bursts.separation*df.clk_p
-        assert separation.min() >= fuse_ms*1e-3
+        separation = bursts.separation * df.clk_p
+        if bursts.num_bursts > 0:
+            assert separation.min() >= fuse_ms * 1e-3
 
 def test_calc_sbr(data):
     """Smoke test Data.calc_sbr()"""
@@ -722,12 +748,15 @@ def test_calc_sbr(data):
 def test_calc_max_rate(data):
     """Smoke test for Data.calc_max_rate()"""
     data.calc_max_rate(m=10)
-    if data.ALEX:
+    if data.alternated:
         data.calc_max_rate(m=10, ph_sel=Ph_sel(Dex='DAem'), compact=True)
 
 def test_burst_data(data):
-    """Smoke test for bext.burst_data()"""
+    """Test for bext.burst_data()"""
     bext.burst_data(data, include_bg=True, include_ph_index=True)
+    bext.burst_data(data, include_bg=False, include_ph_index=True)
+    bext.burst_data(data, include_bg=True, include_ph_index=False)
+    bext.burst_data(data, include_bg=False, include_ph_index=False)
 
 def test_print_burst_stats(data):
     """Smoke test for burstlib.print_burst_stats()"""
@@ -749,6 +778,34 @@ def test_expand(data):
         assert (bg_d == bg_d2).all() and (bg_a == bg_a2).all()
 
 
+def test_burst_data_ich(data):
+    """Test method `Data.burst_data_ich()`."""
+    d = data
+    for ich, bursts in enumerate(d.mburst):
+        if bursts.num_bursts == 0:
+            continue  # if no bursts skip this ch
+        burst_dict = d.burst_data_ich(ich=ich)
+        assert (burst_dict['size_raw'] == bursts.counts).all()
+        assert (burst_dict['t_start'] == bursts.start * d.clk_p).all()
+        assert (burst_dict['t_stop'] == bursts.stop * d.clk_p).all()
+        assert (burst_dict['i_start'] == bursts.istart).all()
+        assert (burst_dict['i_stop'] == bursts.istop).all()
+        assert (burst_dict['bg_period'] == d.bp[ich]).all()
+        nd, na, bg_d, bg_a, width = d.expand(ich, width=True)
+        width_ms = width * 1e3
+        assert (width_ms == burst_dict['width_ms']).all()
+        assert (nd == burst_dict['nd']).all()
+        assert (na == burst_dict['na']).all()
+        assert (bg_d == burst_dict['bg_dd']).all()
+        assert (bg_a == burst_dict['bg_ad']).all()
+        if d.alternated:
+            period = d.bp[ich]
+            bg_da = d.bg_from(Ph_sel(Aex='Dem'))[ich][period] * width
+            bg_aa = d.bg_from(Ph_sel(Aex='Aem'))[ich][period] * width
+            assert (bg_da == burst_dict['bg_da']).all()
+            assert (bg_aa == burst_dict['bg_aa']).all()
+
+
 def test_burst_corrections(data):
     """Test background and bleed-through corrections."""
     d = data
@@ -762,7 +819,7 @@ def test_burst_corrections(data):
         burst_size_raw = bursts.counts
 
         lk = leakage[ich]
-        if d.ALEX:
+        if d.alternated:
             nda, naa = d.nda[ich], d.naa[ich]
             period = d.bp[ich]
             bg_da = d.bg_from(Ph_sel(Aex='Dem'))[ich][period]*width
@@ -799,6 +856,7 @@ def test_burst_search_consistency(data):
         start, stop, width = mb.start, mb.stop, mb.width
         assert np.all(width <= stop - start)
 
+
 def test_E_and_S_with_corrections(data):
     d = data
     gamma = 0.5
@@ -807,8 +865,10 @@ def test_E_and_S_with_corrections(data):
     d.beta = beta
     for i, (E, nd, na) in enumerate(zip(d.E, d.nd, d.na)):
         assert (E == na / (nd * gamma + na)).all()
-        if d.ALEX:
+        if d.alternated:
             naa = d.naa[i]
+            if 'PAX' in data.meas_type:
+                naa = d.naa[i] - d.nar[i]
             assert (d.S[i] == (gamma * nd + na) /
                               (gamma * nd + na + naa / beta)).all()
 
@@ -818,7 +878,7 @@ def test_burst_size_da(data):
     """
     d = data
     d.calc_ph_num(alex_all=True)
-    if d.ALEX:
+    if d.alternated:
         for mb, nd, na, naa, nda in zip(d.mburst, d.nd, d.na, d.naa, d.nda):
             tot_size = mb.counts
             tot_size2 = nd + na + naa + nda
@@ -859,7 +919,7 @@ def test_burst_selection_nocorrections(data):
     assert list_array_equal(ds1.nd, ds2.nd)
     assert list_array_equal(ds1.na, ds2.na)
     assert list_array_equal(ds1.E, ds2.E)
-    if d.ALEX:
+    if d.alternated:
         assert list_array_equal(ds1.naa, ds2.naa)
         assert list_array_equal(ds1.E, ds2.E)
 
@@ -877,7 +937,7 @@ def test_burst_selection_ranges(data):
         time=Range(1, 61, lambda d, ich: d.mburst[ich].start * d.clk_p),
         width=Range(0.5, 1.5, lambda d, ich: d.mburst[ich].width * d.clk_p*1e3),
         peak_phrate=Range(50e3, 150e3, lambda d, ich: d.max_rate[ich]))
-    if d.ALEX:
+    if d.alternated:
         sel_functions.update(naa=Range(30, 40, None), S=Range(0.3, 0.7, None))
 
     for func_name, range_ in sel_functions.items():
@@ -891,6 +951,7 @@ def test_burst_selection_ranges(data):
             selected = getter(ds, ich)
             assert ((selected >= range_.min) * (selected <= range_.max)).all()
 
+
 def test_join_data(data):
     """Smoke test for bext.join_data() function.
     """
@@ -900,6 +961,7 @@ def test_join_data(data):
     for bursts in dj.mburst:
         assert (np.diff(bursts.start) > 0).all()
 
+
 def test_collapse(data_8ch):
     """Test the .collapse() method that joins the ch.
     """
